#ifndef MYX_MATH_UNITS_HPP_
#define MYX_MATH_UNITS_HPP_

#include <type_traits>
#include <cmath>

namespace myx {

namespace math {

/*!
 * \brief radiansToDegrees Перевод из радиан в градусы
 * \param radians          Значение в радианах
 * \return                 Значение в градусах
 */
template < typename T >
T radians_to_degrees( const T& radians, typename std::enable_if< std::is_floating_point< T >::value >::type* = 0 )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( radians * M_1_PI * 180.0 );
}


/*!
 * \brief degreesToRadians Перевод из градусов в радианы
 * \param degrees          Значение в градусах
 * \return                 Значение в радианах
 */
template < typename T >
T degrees_to_radians( const T& degrees, typename std::enable_if< std::is_floating_point< T >::value >::type* = 0 )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( degrees * M_PI / 180.0 );
}


/*!
 * \brief decibel_to_power Перевод из децибелов в мощности
 * \param degrees          Значение в градусах
 * \return                 Значение в радианах
 */


// y = 10^{x / 10}
template < typename T >
T decibel_to_power( T const& db )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( std::pow( 10, db / 10.0 ) );
}


// y = 10 * \log(x)
template < typename T >
T power_to_decibel( T const& power )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( 10 * std::log( power ) );
}


// y = 10^{x / 20}
template < typename T >
T decibel_to_magnitude( T const& value )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( std::pow( 10, value / 20.0 ) );
}


// y = 10^{x / 20}
template < typename T >
T decibel_to_magnitude( T const& value )
{
	static_assert( std::is_arithmetic< T >::value, "Arithmetic type is required." );
	return( std::pow( 10, value / 20.0 ) );
}

} // namespace math

} // namespace myx


#endif // MYX_MATH_UNITS_HPP_
